dojo.provide("ram.assettype.AttributeConstraints");

dojo.require("ram.RamDialog");
dojo.require("dijit.Tree");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.DropDownButton");
dojo.require("jazz.ui.Menu");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.Toolbar");
dojo.require('dojo.data.ItemFileReadStore');
dojo.require('ram.grid.DataGrid');
dojo.require('ram.fragments.TablePagingFiltering');
dojo.require('jazz.ui.ResourceLink');

dojo.requireLocalization("ram", "AssetTypeEditorMessages");

dojo.declare(
		"ram.assettype._AttributeConstraintTreeNode", [ram.assettype._DisplayAttributeConstraintTreeNode, dijit._TreeNode],
{
	templateString: dojo.cache("ram", "assettype/TreeNode.html"),
	
	editButton: null,
	deleteButton: null,
	insertButton: null,
	
	postCreate:function () {
		this.inherited(arguments);
	},
	
	_setIndentAttr: function(indent){
		// summary:
		//		Tell this node how many levels it should be indented
		// description:
		//		0 for top level nodes, 1 for their children, 2 for their
		//		grandchildren, etc.
		this.indent = indent;
		
		// Math.max() is to prevent negative padding on hidden root node (when indent == -1)
		var pixels = ((Math.max(indent, 0) * this.tree._nodePixelIndent) + this.tree._dndGripperIndent) + "px";

		dojo.style(this.domNode, "backgroundPosition",	pixels + " 0px");
		dojo.style(this.rowNode, this.isLeftToRight() ? "paddingLeft" : "paddingRight", pixels);

		dojo.forEach(this.getChildren(), function(child){
			child.set("indent", indent+1);
		});
	},
	
	refreshPresentation:function() {
		this._styleRow();
	},
	
	_createMenuBar: function() {
		var pMenuBar = new dijit.Toolbar();
		
		var type =  this.tree.model.store.getValue(this.item, "type");
      	
		if (type == "attribute_group" || type == "column") {
			var menu = new jazz.ui.Menu({
	            style: "display: none;"
	        });
			
			var props;
			if (type == "attribute_group")
				props = {
					label: this.msg.menuItemAddAttributeGroup,
					disabled: true
		        }
			else 
				props = {
					label: this.msg.menuItemAddAttributeGroup,
					onClick: dojo.hitch(this, "_insertGroup")
	        	}
	        var menuItem1 = new dijit.MenuItem(props);
	        
	        menu.addChild(menuItem1);

	        var menuItem2 = new dijit.MenuItem({
	            label: this.msg.menuItemAddAttributeConstraints
	        });
	        dojo.connect(menuItem2.domNode, "onclick", dojo.hitch(this, "_insertConstraints"));
	        menu.addChild(menuItem2);
	        
	        this.insertButton = new dijit.form.DropDownButton({
	            label: this.msg.buttonInsert,
	            iconClass: "dijitEditorIcon rowActionInsert",
	            dropDown: menu,
	            showLabel: false
	        });
	       
	        pMenuBar.addChild(this.insertButton);
		}
		
		if (type != "ram_attribute" && type != "column") {
			this.editButton = new dijit.form.Button({
	            label: this.msg.buttonEdit,
	            iconClass: "dijitEditorIcon rowActionEdit",
	            showLabel: false
	        });
			pMenuBar.addChild(this.editButton);
			
	        this.deleteButton = new dijit.form.Button({
	            label: this.msg.buttonDelete,
	            iconClass: "dijitEditorIcon rowActionDelete",
	            showLabel: false
	        });
		
	        pMenuBar.addChild(this.deleteButton);
		}
        
        pMenuBar.placeAt(this.menuBarWrapper);
	},
	
	_insertGroup: function() {
		this.tree._insertNode(this.item, "attribute_group");
	},
	_insertConstraints: function() {
		this.tree._insertNode(this.item, "attribute");
	}
});


dojo.declare("ram.assettype._EditAttributeNodeDialogContent", [dijit._Widget, dijit._Templated, dijit._Contained], {
	templateString: dojo.cache("ram", "assettype/EditAttributeNodeDialogContent.html"),
	widgetsInTemplate: true,
	
	item: null,
	model: null,
	users: null,
	userDialogConnection: null,
	
	constructor: function() {
		this.msg = dojo.i18n.getLocalization("ram", "AssetTypeEditorMessages");
	},
	
	postCreate: function(){
		this.inherited(arguments);	
		
		this.users = [];
		
		var store = this.model.store;
		if (store.getValue(this.item, "required")) {
			dojo.attr(this.requiredCheckbox, "checked", "checked");
		}
		
		if (this.item.authorizedUpdateUserIds)
			this.getUserDetails(this.item.authorizedUpdateUserIds);
		
		if (this.users.length > 0)
			this.authUser.checked = true;
		
		this.refreshUserTable();
	},
	
	getUserDetails: function(userIds) {
		for(var i=0; i<userIds.length; i++) {
			dojo.xhrGet({
				url: getAbsoluteUrl("internal/users/" + userIds[i]),
				handleAs: "json",
				headers: {"Accept": "application/json"},
				sync: true,
				load: dojo.hitch(this, function() {
					this.users.push(arguments[0]);
				})
			});
		}
	},
	
	getUpdateUserIds: function() {
		var userIds = [];
		for(var i=0; i<this.users.length; i++) {
			userIds.push(this.users[i].identifier);
		}
		
		return userIds;
	},
	
	editNode: function() {
		var req = this.requiredCheckbox.checked;
		var store = this.model.store;
		var dirty = false;
		if (store.getValue(this.item, "required") != req) {
			store.setValue(this.item, "required", req);
			dirty = true;
		}
		store.setValue(this.item, "authorizedUpdateUserIds", this.getUpdateUserIds());
		store.save();
		this.cancelEdit();
	},
	
	openAddUsersDialog: function() {
		var dialog = dijit.byId("addUserDialog");
		if (!dialog) {
			dialog = new ram.users.UserSelectionDialog({ id:"addUserDialog", helpId: null});
		}
		
		this.userDialogConnection = dojo.connect(dialog, "onAdd", dojo.hitch(this, this.addUsersAndRepaint));
		dojo.connect(dialog, "hide", this, function(){dojo.disconnect(this.userDialogConnection)});
		dialog.show();
	},
	
	addUsers: function(users) {
		for (var i=0; i<users.length; i++) {
			if (!this._containsUser(users[i]))
				this.users.push(users[i]);
		}
	},
	
	_containsUser : function(user) {
		for (var i=0; i<this.users.length; i++) {
			if (this.users[i].identifier == user.identifier)
				return true;
		}
		
		return false;
	},
	
	addUsersAndRepaint: function(users) {
		this.addUsers(users);
		this.refreshUserTable();
	},
	
	refreshUserTable: function() {
		dojo.empty(this.userContainer);
		
		dojo.forEach(this.users, dojo.hitch(this, function(data){
			var row = dojo.create("tr", null, this.userContainer);
			var cell1 = dojo.create("td", null, row);
			var navLink = new jazz.ui.ResourceLink({
				uri: data.base + data.href,
				presentationUri: data.base + data.href,
				lazyFetch: true,
				label: data.name
			});
			dojo.place(navLink.domNode, cell1);
			
			var cell2 = dojo.create("td", {width: "22px"}, row);
			var pMenuBar = new dijit.Toolbar({"class": "InlineToolbar"});
			var removeButton = new dijit.form.Button({
	            label: this.msg.buttonRemove,
	            iconClass: "dijitEditorIcon rowActionRemove",
	            showLabel: false,
	            onClick: dojo.hitch(this, this.removeUser, data)
	        });
			dojo.connect(removeButton, "onclick", dojo.hitch(this, function() {this.removeUser(data)}));
	        pMenuBar.addChild(removeButton);
	        pMenuBar.placeAt(cell2);
		}));
	},
	
	removeUser: function(user) {
		for (var i=0; i<this.users.length; i++) {
			if (this.users[i].identifier == user.identifier)
				this.users.splice(i, 1);
		}
		this.refreshUserTable();
	},
	
	cancelEdit: function() {
		this.getParent().hide();
	}
});

dojo.declare("ram.assettype._NewAttributeGroupDialogContent", [dijit._Widget, dijit._Templated, dijit._Contained], {
	templateString: dojo.cache("ram", "assettype/NewAttributeGroupDialogContent.html"),
	widgetsInTemplate: true,
	
	name: "",
	model: null,
	item: null,
	
	constructor: function() {
		this.msg = dojo.i18n.getLocalization("ram", "AssetTypeEditorMessages");
	},

	postCreate: function(){
		this.inherited(arguments);
		
		if (this.model.store.getValue(this.item, "type") == "attribute_group") {
			// Edit mode
			this.addCloseButton.attr("label", this.msg.buttonOK);
			this.nameField.attr("value", this.model.store.getValue(this.item, "name"));
		}
	},
	
	addNodeAndClose: function() {
		var name = this.nameField.value;
		if (dojo.trim(name).length == 0)
			return; // Don't add - might want to cancel the close if they chose "Add and Close" - might want to show error
		if (this.model.store.getValue(this.item, 'type') == 'column') {
			var id = 'id_' + Math.floor(Math.random() * 100000);
			var newItem = {identifier: id, name: name, type: 'attribute_group', constraints: []};
			this.model.newItem(newItem, this.item);
		} else {
			this.model.store.setValue(this.item, 'name', name);
			this.model.store.save();
		}
		this.nameField.value = '';
		this.cancelEdit();
	},
	
	cancelEdit: function() {
		this.getParent().hide();
	}
});

dojo.declare("ram.assettype._NewAttributeNodeDialogContent", [dijit._Widget, dijit._Templated, dijit._Container, dijit._Contained], {
	templateString: dojo.cache("ram", "assettype/NewAttributeNodeDialogContent.html"),
	widgetsInTemplate: true,
	
	model: null,
	item: null,
	datagrid: null,
	pageSize: 25,
	onDeselectHandle: null,
	onSelectHandle: null,
	selectedAttributes: {}, // map of attribute URIs to attribute objects
	
	constructor: function() {
		this.msg = dojo.i18n.getLocalization("ram", "AssetTypeEditorMessages");
	},

	postCreate: function(){
		this.inherited(arguments);
		
		dojo.xhrGet({
			url: getAbsoluteUrl('internal/attributes'),
			handleAs: 'json',
			headers: {"Accept": "application/json"},
			load: dojo.hitch(this, this.loadTableFragment)
		});
	},
	
	loadTableFragment: function(responseData) {
		for (var i=0; i<responseData.length; i++) {
			responseData[i].description = responseData[i].description ? responseData[i].description : "";
		}
		
		var allItems = new dojo.data.ItemFileReadStore({
			data: {
				identifier: 'identifier',
	            label: 'name',
	            items: responseData
	        }
	    });
		var compareFn = function(a, b) {
			return a.localeCompare(b);
		};
		allItems.comparatorMap = {'name': compareFn, 'description': compareFn}

		var layout = [{
					field: 'name',
					name: this.msg.columnName,
					width: '200px',
					noresize: true
				}, 
				{
					field: 'description',
					name: this.msg.columnDescription,
					width: '400px',
					noresize: true
				}];
		this.datagrid = new ram.grid.DataGrid({rowsPerPage: this.pageSize, autoHeight: this.pageSize, structure: layout});
		this.datagrid.setStore(allItems);

		var table = new ram.fragments.TablePagingFiltering({id: "attributesTable", dataGrid: this.datagrid, store: allItems, rowsPerPage: this.pageSize});
		dojo.connect(table.pager, 'onNextClick', this, 'fixSelection');
		dojo.connect(table.pager, 'onPreviousClick', this, 'fixSelection');
		dojo.connect(table.pager2, 'onNextClick', this, 'fixSelection');
		dojo.connect(table.pager2, 'onPreviousClick', this, 'fixSelection');
		dojo.connect(this.datagrid, 'filter', this, 'fixSelection');
		this.onSelectHandle = dojo.connect(this.datagrid.selection, 'onSelected', this, 'onSelect');
		this.onDeselectHandle = dojo.connect(this.datagrid.selection, 'onDeselected', this, 'onDeselect');

		this.addChild(table);
		this.datagrid.startup();
	},
	
	onSelect: function(index) {
		var item = this.datagrid.getItem(index);
		var uri = this.datagrid.store.getValue(item, 'href');
		this.selectedAttributes[uri] = item;
	},
	
	onDeselect: function(index) {
		var item = this.datagrid.getItem(index);
		var uri = this.datagrid.store.getValue(item, 'href');
		delete this.selectedAttributes[uri];
	},
	
	fixSelection: function() {
		// The reason we need to fix the selection here is because the selected row remains selected after page change/filter, even though the item in that row is different.
		// For example, if you selected "Build ID" at index 3 then clicked next, when the next page loads, the attribute at index 3 will be selected even though it's not "Build ID".

		// disconnect our onSelect/onDeselect functions... we're changing the selection due to a page change/filter, not because the user selected/deselected an attribute
		dojo.disconnect(this.onSelectHandle); 
		dojo.disconnect(this.onDeselectHandle); 
		
		this.datagrid.selection.clear(); // clear the selection
		// go through the items on this page and set the selection based on selectedAttributes
		for (var i = 0; i < this.pageSize; i++) {
			var item = this.datagrid.getItem(i);
			if (!item)
				break; // no more items on the page
			var uri = this.datagrid.store.getValue(item, 'href');
			this.datagrid.selection.setSelected(i, this.selectedAttributes[uri]); // if undefined, deselect; otherwise, select
		}
		
		// reconnect our onSelect/onDeselect functions
		this.onDeselectHandle = dojo.connect(this.datagrid.selection, 'onDeselected', this, 'onDeselect');
		this.onSelectHandle = dojo.connect(this.datagrid.selection, 'onSelected', this, 'onSelect');
	},
	
	getSelected: function() {
		var items = [];
		for (var uri in this.selectedAttributes) {
			if (this.selectedAttributes.hasOwnProperty(uri)) {
				items.push(this.selectedAttributes[uri]);
			}
		}
		return items;
	},
	
	addNodeAndClose: function() {
		var items = this.getSelected();
		
		dojo.forEach(items, dojo.hitch(this, function(entry, i) {
			var newIdentity = this.datagrid.store.getValue(entry, "identifier");
			
			if (this.model.store._itemsByIdentity[newIdentity] === undefined) {
				var desc = this.datagrid.store.getValue(entry, "description");
				var newItem = {type: "constraint", identifier: newIdentity, name: this.datagrid.store.getValue(entry, "name"), description: desc ? desc : "", required: false };
				this.model.newItem(newItem, this.item);
			}
		}));
		
		this.cancelEdit();
	},
	
	cancelEdit: function() {
		this.datagrid.selection.clear();
		this.selectedAttributes = {};
		this.getParent().hide();
	}
});

dojo.declare(
	"ram.assettype.AttributeConstraints",
	[dijit.Tree, dijit._Widget, dijit._Templated],
{		
	_dndGripperIndent: 14,
	
	constructor: function() {
		this.msg = dojo.i18n.getLocalization("ram", "AssetTypeEditorMessages");
	},

	postCreate: function(){
		this.inherited(arguments);
	},
	
	_createTreeNode: function(/*Object*/ args){
		// summary:
		//		creates a TreeNode
		// description:
		//		Developers can override this method to define their own TreeNode class;
		//		However it will probably be removed in a future release in favor of a way
		//		of just specifying a widget for the label, rather than one that contains
		//		the children too.
		
		
		return new ram.assettype._AttributeConstraintTreeNode(args);
	},
	
	_onKeyPress: function(/*Event*/ e){
		if (e.charCode==dojo.keys.SPACE)
			dojo.stopEvent(e);
		else 
			this.inherited(arguments);
	},
	
	checkItemAcceptance: function(target, source, position){
		// summary:
		//		Stub function to be overridden if one wants to check for the ability to drop at the node/item level
		// description:
		//		In the base case, this is called to check if target can become a child of source.
		//		When betweenThreshold is set, position="before" or "after" means that we
		//		are asking if the source node can be dropped before/after the target node.
		// target: DOMNode
		//		The dijitTreeRoot DOM node inside of the TreeNode that we are dropping on to
		//		Use dijit.getEnclosingWidget(target) to get the TreeNode.
		// source: dijit.tree.dndSource
		//		The (set of) nodes we are dropping
		// position: String
		//		"over", "before", or "after"
		// tags:
		//		extension
		
		/* 
		 * The tough case is if position==before or after, target is an attribute constraint, and the source is a attribute group
		 * in this case, we need to know if the target attribute is a child of an attribute group (bad) or a column (good)
		 */
		var sourceType = dijit.getEnclosingWidget(source.anchor.domNode).item.type[0];
		var targetType = dijit.getEnclosingWidget(target).item.type[0];
		var targetParentType = dijit.getEnclosingWidget(target).getParent().item.type[0];
		
		if (sourceType == "column")
			return false;
		
		if (position!="over") {
			if (targetType == "column")
				return false;
			// Reordering care about what the targetParentType is
			if (sourceType == "attribute_group")
				return targetParentType != "attribute_group";
		} else {
			if (sourceType == "attribute_group") {
				return targetType == "column";
			}
			return targetType == "attribute_group" || targetType == "column";
		}
			
		return true;
	},
	
	_onClick: function(/*TreeNode*/ nodeWidget, /*Event*/ e){
		// summary:
		//		Translates click events into commands for the controller to process

		var domElement = e.target,
			isExpandoClick = (domElement == nodeWidget.expandoNode || domElement == nodeWidget.expandoNodeText),
			isEditClick = (nodeWidget.editButton && (domElement == nodeWidget.editButton.valueNode || domElement == nodeWidget.rowActionEditText)),
			isDeleteClick = (nodeWidget.deleteButton && (domElement == nodeWidget.deleteButton.valueNode || domElement == nodeWidget.rowActionDeleteText));
			isInsertClick = (nodeWidget.insertButton && domElement == nodeWidget.insertButton.valueNode);

		
		if ((this.openOnClick && nodeWidget.isExpandable) || isExpandoClick) {
			// expando node was clicked, or label of a folder node was clicked; open it
			if (nodeWidget.isExpandable) {
				this._onExpandoClick({node:nodeWidget});
			}
		} else if (isEditClick) {
			this._editNode(nodeWidget);
		} else if (isDeleteClick) {
			// Special case, stop the event before deleting the element, then return.
			dojo.stopEvent(e);
			this._deleteNode(nodeWidget);
			return;
		} else if (isInsertClick) {
			// Do nothing, just don't want to fall into the else block
		} else {
			this._publish("execute", {item: nodeWidget.item, node: nodeWidget, evt: e});
			this.onClick(nodeWidget.item, nodeWidget, e);
			this.focusNode(nodeWidget);
		}
		if (!isExpandoClick) {
			this._setSelectedNodeAttr(nodeWidget);
		}
		dojo.stopEvent(e);
	},

	_editNode: function(/*TreeNode*/ nodeWidget) {
		var type = nodeWidget.item.type[0];
		var dialog, dialogContent;
		
		if (type== "attribute_group") {
			dialogContent = new ram.assettype._NewAttributeGroupDialogContent({model: this.model, item: nodeWidget.item});
			dialog = new ram.RamDialog({ id:"editDialog", title: this.msg.dialogTitleEditAttributeGroup, helpId: 'raat0075'});
		} else {
			dialogContent =  new ram.assettype._EditAttributeNodeDialogContent({model: this.model, item: nodeWidget.item});
			dialog = new ram.RamDialog({ id:"editDialog", title: this.msg.dialogTitleConstraintSettings, helpId: 'raat0075'});
			dojo.connect(dialogContent, "editNode", nodeWidget, function(e){nodeWidget.refreshPresentation()});
		}
		
		dialog.attr("content", dialogContent);
		dialog.show();
	
		dojo.connect(dialog, "hide", dialog, function(e){dialog.destroyRecursive(false)})
	},
	
	_insertNode: function(/*dojo.data.Item*/ item, /*String*/ type) {
		var dialog, dialogContent;
		var oldUnselect;
		//modify css here to stop the IE10 user select
		if(document.body.style.setProperty){
			document.body.style.setProperty('-ms-user-select', 'none', null);
		}else{
			document.body.style.setAttribute('-ms-user-select', 'none', null);
		}
		
		oldUnselect = document.body.getAttribute('unselectable'); 
		document.body.setAttribute('unselectable', 'on');
			
		if (type == "attribute"){
			dialog = new ram.RamDialog({ id:"editDialog", title: this.msg.dialogTitleSelectAttributes, helpId: "raat0070"});
			dialogContent =  new ram.assettype._NewAttributeNodeDialogContent({model: this.model, item: item});
		} else if (type== "attribute_group") {
			dialog = new ram.RamDialog({ id:"editDialog", title: this.msg.dialogTitleNewAttributeGroup, helpId: "raat0060"});
			dialogContent = new ram.assettype._NewAttributeGroupDialogContent({model: this.model, item: item});
		}
		
		dialog.attr("content", dialogContent);
		dialog.show();
		
		dojo.connect(dialog, "hide", dialog, function(e){dialog.destroyRecursive(false);
			//remove the IE10 user select feature.
			if (document.body.style.removeProperty) {
				document.body.style.removeProperty ('-ms-user-select');
	        } 
	        else {
	        	document.body.style.removeAttribute ('-ms-user-select');
	        }
			if(oldUnselect != null){
				document.body.setAttribute('unselectable', oldUnselect);
			}else{
				document.body.removeAttribute ('unselectable');
			}
		})
	},
	
	_deleteNode: function(/*TreeNode*/ nodeWidget) {
		var item = nodeWidget.item;
		if (this.model.store.getValue(item, "type") == "attribute_group") {
			this.model.getChildren(item, dojo.hitch(this, function(nodeWidget, children){
				var parent = nodeWidget.getParent().item;
				dojo.forEach(children, dojo.hitch(this,  function(entry, i) {
					if (this.model.store.getValue(entry, "type") == "ram_attribute")
						this.model.pasteItem(entry, item, parent, false); // Move RAM attribute children before deleting group and its children
				 }));
			}, nodeWidget));
		}
		this.model.store.deleteItem(item);
	}
});