dojo.provide('ram.ui.PathBar');
dojo.require('dijit._Templated');
dojo.require('dijit._Widget');
dojo.require('dijit._Container');
dojo.require('dijit._Contained');

dojo.declare(
	'ram.ui._PathSegment',
	[dijit._Widget, dijit._Templated, dijit._Contained],
{
	templateString: '<span style="color: #316AC5; vertical-align: top;">&lrm;/<a style="cursor: pointer; color: #316AC5; text-decoration: none;" title="${name}" dojoattachevent="onclick:_navigate" dojoattachpoint="link">...</a></span>',
	
	name: '',
	path: '',
	
	_navigate: function() {
		this.getParent().setPath(this.path);
	},
	
	expand: function() {
		this.link.innerHTML = this.name;
		this.link.title = '';
	},
	
	collapse: function() {
		this.link.innerHTML = '...';
		this.link.title = this.name;
	}
	
});

dojo.declare(
	'ram.ui.PathBar',
	[dijit._Widget, dijit._Templated, dijit._Container],
{
	templateString: null,
	templatePath: dojo.moduleUrl('ram.ui', 'templates/PathBar.html'),
	
	path: '',
	icon: null,
	
	postCreate: function() {
		this.iconNode.src = this.icon ? this.icon : getAbsoluteUrl('theme/images/icons/obj16/asset.gif');
		this.setPath(this.path);
	},
	
	_renderPath: function() {
		dojo.style(this.containerNode, 'visibility', 'hidden');
		this._clear();
		var segments = dojo.filter(this.path.split('/'), function(item) {
			return item.length > 0;
		});
		if (segments.length > 0) {
			for (var i = 0; i < segments.length; i++) {
				if (segments[i].length > 0) {
					var folderPath = '/' + segments.slice(0, i + 1).join('/');
					this.addChild(new ram.ui._PathSegment({name: segments[i], path: folderPath}));
				}
			}
		} else {
			this.addChild(new ram.ui._PathSegment());
		}
		this._expandSegments();
		dojo.style(this.containerNode, 'visibility', 'visible');
	},
	
	_expandSegments: function() {
		// Segment labels are initially ... so we can determine if the path is too long to fit in the container.
		// We expand segments until the width of the path bar exceeds its max width, then shrink the last segment to be expanded so everything fits.
		var children = this.getChildren();
		var currentIndex = children.length - 1;
		var nextIndex = 0;
		var increment = -1;
		while (true) {
			// get the width before expanding the current segment. this is the max width.
			var beforeWidth = dojo.position(this.domNode).w;
			var segment = children[currentIndex];
			segment.expand();
			// get the width after expanding the current segment.
			var afterWidth = dojo.position(this.domNode).w;

			// if the width has increased over the max width, collapse the current segment
			if (afterWidth > beforeWidth) {
				segment.collapse();
				break;
			}
			
			if (currentIndex == nextIndex || nextIndex == -1)
				break;
			
			var tempIndex = currentIndex;
			currentIndex = nextIndex;
			nextIndex = tempIndex + increment;
			increment = -increment;
		}
	},
	
	_clear: function() {
		var children = this.getChildren();
		for (var i = 0; i < children.length; i++) {
			this.removeChild(children[i]);
		}
	},
	
	setPath: function(path) {
		if (!path)
			this.path = '';
		else
			this.path = path;
		this.navigate(this.path);
		this._renderPath();
	},
	
	appendToPath: function(path) {
		if (path && dojo.isString(path)) {
			if (!path.startsWith('/'))
				path = '/' + path;
			this.path = this.path + path;
			this.navigate(this.path);
			this._renderPath();
		}
	},
	
	_goToRoot: function() {
		this.setPath('');
	},
	
	refresh: function() {
		this.setPath(this.getPath());
	},
	
	getPath: function() {
		var p = this.path;
		if (p.length >= 0 && !p.startsWith('/'))
			p = '/' + p;
		return p;
	},
	
	navigate: function(path) {}
	
});
