dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("ram.relationshiptree.TreeStoreModel");
dojo.require("ram.relationshiptree.EditorRelationshipTree");
dojo.require("dijit.form.Button");
dojo.require("jazz.ui.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.TitlePane");
dojo.require("jazz.ui.StyledBox");
dojo.require("dojo.NodeList-traverse");

var CATEGORY_SCHEMA_URL = "submitServlet/schemas";
var ASSET_TYPES_URL = "submitServlet/assetTypes";
var RELATED_MESSAGES_URL = "submitServlet/related/messages";
var CONTENT_MESSAGES_URL = "submitServlet/content/messages";
var RELATIONSHIP_TREE_URL = "internal/service/SubmitAssetHandlerService.getRelatedAssets";
var RELATIONSHIP_TYPES_URL = "internal/service/SubmitAssetHandlerService.getAvailableRelationships";
var RELATIONSHIP_SUGGESTIONS_URL = "internal/service/SubmitAssetHandlerService.getSuggestedRelationships";

var IMG_CATEGORY_SCHEMA = "/theme/images/icons/obj16/category_schema.gif";
var IMAGE_EXPAND = "/theme/images/icons/elcl16/expand-tree.gif";
var IMAGE_COLLAPSE = "/theme/images/icons/elcl16/collapse-tree.gif";

var IMG_RADIO_SELECTED = getAbsoluteUrl("theme/images/icons/elcl16/radio-full-select.gif");
var IMG_RADIO_UNSELECTED = getAbsoluteUrl("theme/images/icons/elcl16/radio-none-select.gif");
var IMG_RADIO_CHILD_SELECTED = getAbsoluteUrl("theme/images/icons/elcl16/radio-part-select.gif");
var IMG_CHECKBOX_SELECTED = getAbsoluteUrl("theme/images/icons/elcl16/check-full-select.gif");
var IMG_CHECKBOX_UNSELECTED = getAbsoluteUrl("theme/images/icons/elcl16/check-none-select.gif");
var IMG_CHECKBOX_CHILD_SELECTED = getAbsoluteUrl("theme/images/icons/elcl16/check-part-select.gif");

var IMG_ERROR = getAbsoluteUrl("theme/images/icons/obj16/error.gif");
var IMG_INFO =  getAbsoluteUrl("theme/images/icons/obj16/info_obj.gif");

var IMG_RADIO_SELECTED_ALT = "selected radio";
var IMG_RADIO_UNSELECTED_ALT = "unselected radio";
var IMG_RADIO_CHILD_SELECTED_ALT = "A child of this radio is selected";
var IMG_CHECKBOX_SELECTED_ALT = "selected checkbox";
var IMG_CHECKBOX_UNSELECTED_ALT = "unselected checkbox";
var IMG_CHECKBOX_CHILD_SELECTED_ALT = "A child of this checkbox is selected";

var HIDDEN_INPUT_ID = "assetUploadForm:selectedCategories";
var PREVIEW_CONTAINER_ID = "categoryOutputContainer";

var PREVIEW_TREE_STYLE = "output_categorizeSchemaList";

var tooltip = null;

var relationshipTree = null;
var relationshipTypes = null;

var relationshipTypeToAdd = null;

/***************************************************
 *	Creates the popup for asset types and communities
 ************Read if using this function************
 *	The title and description will be encoded in the
 *	function. Do not pass in strings that are already 
 *	encoded or else double encoding will occur.
 ***************************************************/
function setToolTip(nodeId, description, title) {
	var hover = dijit.byId('DescriptionHoverPopup');
	if (hover)
		hover.destroy();
	if (description != null && description != '') {
		var tooltipMessage = new StringBuffer();
		if (title != '' && title != null) {
			tooltipMessage.append('<h1>');
			tooltipMessage.append(encodeHtml(title));
			tooltipMessage.append('</h1>');
			tooltipMessage.append('<div class="dividerWrapper"><div class="divider" dojoattachpoint="divider"></div></div>');
		}
		tooltipMessage.append('<span>');
		tooltipMessage.append(encodeHtml(description));
		tooltipMessage.append('</span>');
		hover = new jazz.ui.HoverPopup({
			id: 'DescriptionHoverPopup',
			around: nodeId,
			html: tooltipMessage.toString()
		});
		hover.maxWidth = 400;
		return hover;
	}
	return null;
}

/***************************************************
 *	Padding is required when a SELECT element has
 *	enough OPTION childred to cause a scroll bar to
 *	appear. If no padding is applied the popup will
 *	be displayed behind the scroll bar. Tried
 *	changing the z-index with ro result.
 ***************************************************/
function padToolTip(target){
	if(target.nodeName =='OPTION' && target.parentNode.scrollHeight > target.parentNode.clientHeight){
		var style = dojo.byId('jazz_ui_internal__MasterPopup_0').getAttribute('style');
		style += 'padding-left:20px;';
		dojo.byId('jazz_ui_internal__MasterPopup_0').setAttribute('style',style);
	}
}


/***************************************************
 *	The padding needs to be removed when 
 * 	the popup is closed because it will persist
 *	to other elements that do not require the 
 *	padding.
 ***************************************************/
function removePadding(){
	var patt=new RegExp('padding-left:.*;','');
	var style = dojo.byId('jazz_ui_internal__MasterPopup_0').getAttribute('style');
	style = style.replace(patt, '');
	dojo.byId('jazz_ui_internal__MasterPopup_0').setAttribute('style',style);
	tooltip.close();
}

/***************************************************
 *	Shows the description for 
 *	an asset type when it is hovered over.
 ***************************************************/
function showAssetTypeDescription(event) {
	var target;
	if (event.target)
		target = event.target;
	else if (event.srcElement)
		target = event.srcElement;
	var description = typeMap[target.value];
	tooltip = setToolTip('assetUploadForm:assetTypes', description, null); 
	if (tooltip != null) {
			tooltip.open(target);
			padToolTip(target);
	}
}

/***************************************************
 *	Hides the description of an 
 *	asset type when the mouse leaves the element.
 ***************************************************/
function hideAssetTypeDescription() {
	if(tooltip != null)
		removePadding();
}

/***************************************************
 *	Shows the description for 
 *	a community when it is hovered over.
 ***************************************************/
function showCommunityDescription(event) {
	var target;
	if (event.target)
		target = event.target;
	else if (event.srcElement) // IE
		target = event.srcElement;
	var description = communityMap[target.value];
	tooltip = setToolTip('assetUploadForm:groupname', description, null); 
	if (tooltip != null) {
		tooltip.open(target);
		padToolTip(target);
	}
}

/***************************
 *	Hides the description of a 
 *	community when the mouse leaves the element.
 **************************/
function hideCommunityDescription() {
	if(tooltip != null)
		removePadding();
}

function getCommunityId() {
	var groupInput = dojo.byId('assetUploadForm:groupname'); // submit wizard
	if (groupInput) 
		return groupInput.value;
}

/*****************************
 * Functions for managing the tabs on the submit page
 *****************************/
function startRefreshTabs() {
	var tabs = dojo.byId('submit-tabs').childNodes;
	var currentTabId = "";
	for (var i=0; i<tabs.length; i++) {
		if (tabs[i].tagName=="LI" && tabs[i].className=="current") {
			currentTabId = tabs[i].id;
		}
	}
	
	if ("categories-tab" == currentTabId || "rel-tab" == currentTabId) {
		dojo.byId('submit-content').style.display = "none";
		dojo.byId('assetUploadForm:submit-advanced-content').style.display = "none";
		dojo.byId('submit-description').style.display = "none";
		dojo.byId('submit-categories').style.display = "none";
		dojo.byId('submit-related-assets').style.display = "none";
		
		dojo.byId('submit-loading').style.display = "block";
	}

	saveRelatedAssets();
}

function completeRefreshTabs() {
	var currentTabIndex = dojo.byId('assetUploadForm:current-tab-hidden').value;
	var tabId;

	switch ((currentTabIndex*1)) {
		case 1:
			tabId = "description-tab";
			break;
		case 2:
			tabId = "categories-tab";
			break;
		case 3:
			tabId = "rel-tab";
			break;
		case 4:
			tabId = "submit-advanced-content";
			checkJava();
			break;
		default:
			tabId = "content-tab";
			break;
	}
	showCurrentTab(tabId);
}

function setAssetNameField(fileName) {
	var current = dojo.byId("assetUploadForm:assetNameInputText").value;
	if (!current || current.length==0) {
		if (fileName.lastIndexOf('.')>0) {
			fileName = fileName.substring(0, fileName.lastIndexOf('.'));
		}
		dojo.byId("assetUploadForm:assetNameInputText").value = fileName;
	}
}

function selectTab(tabId) {
	var currentTabId = "";
	var tabs = dojo.byId('submit-tabs').childNodes;
	for (var i=0; i<tabs.length; i++) {
		if (tabs[i].tagName=="LI" && tabs[i].className=="current") {
			tabs[i].className = "";
			currentTabId = tabs[i].id;
		}
	}
	dojo.byId(tabId).className = "current";
	
	showCurrentTab(tabId, currentTabId);
}

function showCurrentTab(tabId, currentTabId) {
	var sectionIdToShow = "";
	var currentTabIndex;
	if ("content-tab" == tabId) {
		sectionIdToShow = "submit-content";
		currentTabIndex = 0;
	} else if ("description-tab" == tabId) {
		sectionIdToShow = "submit-description";
		currentTabIndex = 1;
	} else if ("categories-tab" == tabId) {
		sectionIdToShow = "submit-categories";
		currentTabIndex = 2;
		getCategorySchemas('categoryInputContainer');
	} else if ("rel-tab" == tabId) {
		sectionIdToShow = "submit-related-assets";
		currentTabIndex = 3;
		fetchAvailableRelationshipTypes();
		fetchRelationshipTree();
		fetchSuggestedRelationships();
	} else if ("submit-advanced-content" == tabId) {
		sectionIdToShow = "assetUploadForm:submit-advanced-content";
		currentTabIndex = 4;
	}
	
	dojo.byId('assetUploadForm:current-tab-hidden').value = currentTabIndex;
	
	
	if (currentTabId) {
		if ("content-tab" == currentTabId) {
			showingSection = "submit-content";
		} else if ("description-tab" == currentTabId) {
			showingSection = "submit-description";
		} else if ("categories-tab" == currentTabId) {
			showingSection = "submit-categories";
		} else if ("rel-tab" == currentTabId) {
			showingSection = "submit-related-assets";
		} else if ("submit-advanced-content" == currentTabId) {
			showingSection = "assetUploadForm:submit-advanced-content";
		}
		
		dojo.byId('submit-loading').style.height = dojo.byId(showingSection).scrollHeight + "px"; // might have to be clientHeight, need to test on IE
	}

	getContentMessages();
	getRelatedAssetsMessages();
	
	dojo.style("submit-loading", "display", "none");
	dojo.style("submit-content", "display", "none");
	dojo.style("assetUploadForm:submit-advanced-content", "display", "none");
	dojo.style("submit-description", "display", "none");
	dojo.style("submit-categories", "display", "none");
	dojo.style("submit-related-assets", "display", "none");
	
	dojo.style(sectionIdToShow, "display", "block");
}

function showCurrentTabByNumber() {
	var currentTabIndex = dojo.byId('assetUploadForm:current-tab-hidden').value;
	var tabId;
	var sectionIdToShow;

	switch ((currentTabIndex*1)) {
		case 1:
			sectionIdToShow = "submit-description";
			tabId = "description-tab";
			break;
		case 2:
			sectionIdToShow = "submit-categories";
			tabId = "categories-tab";
			getCategorySchemas('categoryInputContainer');
			break;
		case 3:
			sectionIdToShow = "submit-related-assets";
			fetchAvailableRelationshipTypes();
			fetchRelationshipTree();
			fetchSuggestedRelationships();
			tabId = "rel-tab";
			break;
		case 4:
			sectionIdToShow = "assetUploadForm:submit-advanced-content";
			tabId = "content-tab";
			checkJava();
			break;
		default:
			sectionIdToShow = "submit-content";
			tabId = "content-tab";
			break;
	}

	// ASSUME A TAB IS NOT SET
	dojo.byId(tabId).className = "current";
	dojo.byId(sectionIdToShow).style.display = "block";

	getContentMessages();
	getRelatedAssetsMessages();
}

function shouldAdvancedContentAjax() {
	return !dojo.byId("assetUploadForm:submit-advanced-content-renderer");
}

/***********************
 * Functions for managing attributes (RAM attributes and custom)
 ***********************/
function containsLeadingZero(s) {
	return (s.length > 1 && s.charAt(0)=='0');
}
function validateVersion(field) {
	// Note: Whenever this code is changed then com.ibm.ram.internal.common.util.Utilities.isVersionFormat() must be changed too since there is a version format checker in there also.
	var vArray = field.value.split('.');
	
	var value;
	var upto = vArray.length > 1 ? vArray.length-1 : vArray.length;
	for (var i=0; i<upto; i++) {
		value = vArray[i]*1;
		if (isNaN(value) || value<0) {
			dojo.byId('versionWarning').style.display = "block";
			return;
		}
	}
	
	dojo.byId('versionWarning').style.display = "none";
}

var typeMap = new Array();

function getAssetTypes() {	
	dojo.xhrGet({
		url: getAbsoluteUrl(ASSET_TYPES_URL),
		content: {gid: dojo.byId("assetUploadForm:groupname").value},
		handleAs: "json",
		sync: true, // Defect 40647: sync so that the types menu is updated before the next ajax request is made
		
		// The LOAD function will be called on a successful response.
		load: function(response, ioArgs) {
			var menu = dojo.byId("assetUploadForm:assetTypes");
			var currentValue = menu.value;
			dojo.empty(menu);
			typeMap = new Array();
			for(i in response) {
				menu.options[i] = new Option(response[i].label, response[i].value.id);
				typeMap[response[i].value.id] = response[i].value.description;
				if (response[i].value.id == currentValue) {
					menu.options[i].selected = true;
				}
			}
		}
	});
}

function getMoreAttributesTable() {
	var nl = dojo.query('table[id^="assetUploadForm:moreAttributesTable"]');
	if (nl.length > 0)
		return nl[0];
}
function showMoreAttributes() {
	dojo.xhrGet({
		url: getAbsoluteUrl('internal/service/UserPreferencesService.showMoreAttributesSection?show=true')
	});
	var moreAttrTable = getMoreAttributesTable();
	if (moreAttrTable)
		dojo.style(moreAttrTable, 'display', '');
	dojo.style(dojo.byId('assetUploadForm:lessAttrs'), 'display', '');
	dojo.style(dojo.byId('assetUploadForm:moreAttrs'), 'display', 'none');
}
function showLessAttributes() {
	dojo.xhrGet({
		url: getAbsoluteUrl('internal/service/UserPreferencesService.showMoreAttributesSection?show=false')
	});
	var moreAttrTable = getMoreAttributesTable();
	if (moreAttrTable)
		dojo.style(moreAttrTable, 'display', 'none');
	dojo.style(dojo.byId('assetUploadForm:lessAttrs'), 'display', 'none');
	dojo.style(dojo.byId('assetUploadForm:moreAttrs'), 'display', '');
}

function hideMoreAttributesTable() {
	// The JWL date and spinner controls don't work if the more table's style is 
	// "display: none".  Also, the user search dialog doesn't work in IE if the
	// more table's style is "position: absolute; z-index: -1".  So we initially 
	// show the table and once the page loads, we set it to "display: none".
	var table = getMoreAttributesTable();
	if (table) {
		var hasRequired = dojo.query('.required', table).length > 0;
		var keepOpen = dojo.byId('assetUploadForm:showMoreAttributesSection').value == 'true';
		if (!hasRequired && !keepOpen) { // if there are required attributes in the more section, don't hide it
			dojo.style(table, 'display', 'none');
		} else {
			dojo.style(dojo.byId('assetUploadForm:lessAttrs'), 'display', '');
			dojo.style(dojo.byId('assetUploadForm:moreAttrs'), 'display', 'none');
		}
	}
}

function setAddClicked(link) {
	var hiddenNodes = dojo.query(link).siblings('input[type=hidden]');
	for (var i=0; i<hiddenNodes.length; i++) {
		if (hiddenNodes[i].name.indexOf("text_CLICKED_HEDDEN_INPUT") != -1)
			break;
	}
	var inputNodes = dojo.query(link).siblings('input[type=text]');
	if (inputNodes[0].value != "")
		hiddenNodes[i].value = "true";
	else
		hiddenNodes[i].value = "false";
}

/***********************
 * Functions for categorization
 ***********************/
function getCategorySchemas(resultId) {	
	dojo.xhrGet({
		url: getAbsoluteUrl(CATEGORY_SCHEMA_URL),
		handleAs: "json",
		
		// The LOAD function will be called on a successful response.
		load: function(response, ioArgs) {
			var schemas = response;
			//var tree = "";
			
			var expand = getAbsoluteUrl(IMAGE_EXPAND);
			var collapse = getAbsoluteUrl(IMAGE_COLLAPSE);
			var schemaImg = getAbsoluteUrl("theme/images/icons/obj16/category_schema.gif");
			
			// Clear the loading
			dojo.byId(resultId).innerHTML = "";
			
			for (var i=0; i<schemas.length; i++) {
				var levelId = schemas[i].name + i;
				var tree = new ram.categorytree.CategoryTree({levelId:levelId, description:schemas[i].description, name:schemas[i].name, expandImageUrl: expand, collapseImageUrl: collapse, schemaImageUrl: schemaImg});
				tree.setSchema(schemas[i]);
				dojo.byId(resultId).appendChild(tree.domNode); 
				
				tree.startup();

				
				dojo.connect(tree, "_getCategories", connectCategorySchema);
			}
			
			CategorizeTree_attachListener('assetUploadForm');
			
			return response;
		},
		
		// The ERROR function will be called in an error case.
		error: function(response, ioArgs) {
			console.error("HTTP status code: ", ioArgs.xhr.status);
			return response;
		}
	});
}

function connectCategorySchema(evt) {
	var uris = dojo.byId("assetUploadForm:selectedCategories").value.split(";");
	for (var i=0; i<uris.length; i++) {
		if (uris[i].substring(0, uris[i].indexOf('#'))==this.uri && dojo.byId(uris[i]+"_image")) {
			// Only work on nodes for the schema that was expanded
			selectSchemaNode(dojo.byId(uris[i]+"_image"));
		}
	}
}

function selectSchemaNode(element) {
	switch (CategorizeTree_getValue(element)) {
		case RADIO_UNSELECTED:
			CategorizeTree_switchTo(element, RADIO_SELECTED, IMG_RADIO_SELECTED, IMG_RADIO_SELECTED_ALT);
			CategorizeTree_deselectSiblingsAndNephews(element);
			CategorizeTree_selectParents(element);
			return true;
		case CHECKBOX_UNSELECTED:
			CategorizeTree_switchTo(element, CHECKBOX_SELECTED, IMG_CHECKBOX_SELECTED, IMG_CHECKBOX_SELECTED_ALT);
			CategorizeTree_selectParents(element);
			return true;
	}
}

/*******************************
 * Functions for managing content section
 *******************************/
function fixOverflow() {
	var nav = dojo.byId('nav');
	var offscreen = dojo.byId('offscreenNav');
	var table = dojo.byId('offscreenTable');
	try {
		table.width = nav.offsetWidth;  //causing error in IE7
	} catch (e) {
		table.width = 500;
	}

	var offscreenLinks = offscreen.getElementsByTagName('a');
	var realLinks = nav.getElementsByTagName('a');
	if (realLinks.length == 0) {
		nav.style.visibility = 'visible';
		return;
	}
	var currentIndex = realLinks.length - 1;
	var nextIndex = 0;
	var increment = -1;
	
	while (true) {
		var beforeWidth = offscreen.offsetWidth;
		var offscreenLink = offscreenLinks[currentIndex];
		var offscreenSpans = offscreenLink.getElementsByTagName('span');
		if (offscreenSpans && offscreenSpans[0])
			offscreenSpans[0].innerHTML = offscreenLink.title;
		var afterWidth = offscreen.offsetWidth;

		if (beforeWidth != afterWidth) {
			nav.style.visibility = 'visible';
			return;
		}
		
		var realLink = realLinks[currentIndex];
		var realSpans = realLink.getElementsByTagName('span');
		if (realSpans && realSpans[0]) {
			realSpans[0].innerHTML = realLink.title;
			realLink.title = '';
		}
		
		if (currentIndex == nextIndex || nextIndex == -1) {
			nav.style.visibility = 'visible';
			return;
		}
		var tempIndex = currentIndex;
		currentIndex = nextIndex;
		nextIndex = tempIndex + increment;
		increment = -increment;
	}
}
function onFolderClick(folderPath) {
	var currentPath = dojo.byId('assetUploadForm:artifactFragment:currentPath');
	currentPath.value = folderPath;
	return true;
}
function updateFileUploadControls(showId, hideId) {
	var hidden = dojo.byId('assetUploadForm:artifactFragment:fileUploadSection');
	if (hidden)
		hidden.value = showId;
	var show = dojo.byId(showId);
	if (show)
		show.style.display = 'block';
	var hide = dojo.byId(hideId);
	if (hide)
		hide.style.display = 'none';
	resizeFileUploadControl();
}
function openSection(sectionId) {
	var currentSection = dojo.byId('assetUploadForm:artifactFragment:currentOpenSection');
	var currentId = currentSection.value;
	if (sectionId != currentId) {
		currentSection.value = sectionId;
		var current = dojo.byId(currentId);
		if (current)
			current.style.display = 'none';
		var section = dojo.byId(sectionId);
		if (section)
			section.style.display = 'block';
	}
	return false;
}
function toggleCache() {
	var checkbox = 	dojo.byId('assetUploadForm:artifactFragment:cacheCheckbox');
	var urlSection = dojo.byId('urlSectionContent');
	var cacheSection = dojo.byId('cacheSectionContent');
	
	if (checkbox.checked) {
		urlSection.style.height = '300px';
		cacheSection.style.display = 'block';
	} else {
		urlSection.style.height = '175px';
		cacheSection.style.display = 'none';
	}
	return true;
}
function toggleConnection(newConnection) {
	var connectionSelection = dojo.byId('assetUploadForm:artifactFragment:cacheArtifactCredential');
	var existingConnectionSection = dojo.byId('cacheArtifactDialogExistingCreds');
	var newConnectionSection = dojo.byId('cacheArtifactDialogNewCreds');
	
	if (newConnection) {
		newConnectionSection.style.display = 'block';
		existingConnectionSection.style.display = 'none';
		connectionSelection.selectedIndex = 0;
	} else {
		existingConnectionSection.style.display = 'block';
		newConnectionSection.style.display = 'none';
	}
	return true;
}
function clearFilter() {
	var inputField = dojo.byId('assetUploadForm:artifactFragment:nameFilter');
	inputField.value = '';
	return true;
}
function resizeFileUploadControl() {
	var section = dojo.byId('fileSection');
	var input = dojo.byId('assetUploadForm:artifactFragment:fileArtifact');
	if (section && input) {
		while (section.clientWidth < input.clientWidth) {
			input.size -= 5;
		}
	}
}
function initializeSections() {
	var section;
	var sectionId = dojo.byId('assetUploadForm:artifactFragment:currentOpenSection').value;
	if (sectionId)
		section = dojo.byId(sectionId);
	if (section)
		section.style.display = 'block';
	else {
		sectionId = 'fileSectionContent';
		section = dojo.byId(sectionId);
		if (section)
			section.style.display = 'block';
	}
	var fileUploadSectionId = dojo.byId('assetUploadForm:artifactFragment:fileUploadSection').value;
	var fileUploadSection;
	if (fileUploadSectionId)
		fileUploadSection = dojo.byId(fileUploadSectionId);
	if (fileUploadSection)
		fileUploadSection.style.display = 'block';
	else {
		if (navigator.javaEnabled()) {
			dojo.byId('advancedFileUpload').style.display = 'block';
		} else {
			dojo.byId('assetUploadForm:artifactFragment:basicFileUpload').style.display = 'block';
		}
	}
	resizeFileUploadControl();
	toggleCache();
}
function setEditCredentialName() {
	var credential = dojo.byId('assetUploadForm:artifactFragment:cacheCredential');
	var editCredName = dojo.byId('assetUploadForm:artifactFragment:editCredentialName');
	
	editCredName.value = credential.options[credential.selectedIndex].innerHTML;
	return true;
}
function checkCreateNewCredentialDialog(thisObj) {
	if( thisObj.options[thisObj.selectedIndex].value == "NEW" ) {
		return true;
	} 
	return false;
}
function checkCreateNewCredential(thisObj) {
	if( thisObj.options[thisObj.selectedIndex].value == "NEW" ) {
		dojo.byId('assetUploadForm:artifactFragment:cacheEditCredentialLink').style.display = 'none';
		dojo.byId('assetUploadForm:artifactFragment:cacheDeleteCredentialLink').style.display = 'none';
		return true;
	} else if(thisObj.options[thisObj.selectedIndex].value != "NONE") {
		dojo.byId('assetUploadForm:artifactFragment:cacheEditCredentialLink').style.display = 'inline';
		dojo.byId('assetUploadForm:artifactFragment:cacheDeleteCredentialLink').style.display = 'inline';
	} else {
		dojo.byId('assetUploadForm:artifactFragment:cacheEditCredentialLink').style.display = 'none';
		dojo.byId('assetUploadForm:artifactFragment:cacheDeleteCredentialLink').style.display = 'none';
	}
	return false;
}
function setAssetCacheHiddenFields(artifactName, currentPath) {
	dojo.byId('assetUploadForm:artifactFragment:addCacheArtifactName').value = artifactName;
	dojo.byId('assetUploadForm:artifactFragment:addCacheArtifactPath').value = currentPath;
}
function checkJava() {
	if (navigator.javaEnabled()) {
		var appletDiv = dojo.byId('appletDiv');
		if (appletDiv) {
			var appletExists = false;
			var nodeChildren = appletDiv.childNodes;
			if(nodeChildren){
				for(var i = 0; i < nodeChildren.length; i++){
					   if(nodeChildren.item(i).id == 'upload'){
					       appletExists = true;
					       break;
					   }
				}
			}
			if(appletExists == false){
				var alt_java_disabled='This browser does not support applets';
				appletDiv.innerHTML = "<applet id='upload' name='upload' MAYSCRIPT code='com/ibm/ram/applet/upload/RAMUpload2.class' codebase='"+baseUrl+"/applets/' archive='upload.jar' width='704' height='225' align='middle' alt='"+alt_java_disabled+"'><param name='java_arguments' value='-Djnlp.packEnabled=true'><param name='locale' value='"+pageLocale+"'></applet>";
			}
			if('visible' != appletDiv.style.visibility)
				appletDiv.style.visibility = 'visible';
			
		}
	} else {	
		// Check version manually.  Need to do this because isMinVersion does not always work correctly. 
		var version = "unknown";
		updateFileUploadControls('assetUploadForm:artifactFragment:basicFileUpload', 'advancedFileUpload');
		
		var versionMsg = dojo.byId('javaVersionMsg');
		if (versionMsg) {
			versionMsg.innerHTML = version;
		}
		var appletDiv = dojo.byId('appletDiv');
		if (appletDiv) {
			appletDiv.style.display = 'none';
		}
		var getJavaDiv = dojo.byId('getJavaDiv');
		if (getJavaDiv) {
			getJavaDiv.style.display = 'block';
		}
	}
}

/**********************************
 * Functions for the related assets tab
 **********************************/
var previousAssetType = "";
function hasAssetTypeChange() {
	if (previousAssetType != dojo.byId("assetUploadForm:assetTypes").value) {
		previousAssetType = dojo.byId("assetUploadForm:assetTypes").value;
		return true;
	}
	return false;
}

function fetchRelationshipTree() {
	if (hasAssetTypeChange() || relationshipTree == null)
		dojo.xhrGet({
			url: getAbsoluteUrl(RELATIONSHIP_TREE_URL),
			handleAs: "json",
			sync: false,
			load: loadRelationshipTree
		});
}

function loadRelationshipTree(response, ioArgs) {
	if (relationshipTree != null) {
		relationshipTree.destroyRecursive();
	}
	var store = new dojo.data.ItemFileWriteStore({
		data: {
            label: 'name',
            items: [{ name:"Root", type:"root", constraints:response}]
        },
        typeMap: {
        	"com.ibm.ram.common.data.RelationshipInfo": {
        		type: RelationshipInfo,
        		deserialize: function(item) {return new RelationshipInfo(item)},
        		serialize: function(item) {return new RelationshipInfo(item)}
        	}
        }
    });

	store._saveEverything = updateRelationshipInput;
	
    treeModel = new ram.relationshiptree.TreeStoreModel({
        store: store,
        query: {
    		name: "Root"
        },
        childrenAttrs: ["assetsInRange", "relatedAssets", "constraints"]
    });
    
    dojo.create("div", {id: "relationshipTree"}, dojo.byId("relationshipTreeDiv"));
    
    relationshipTree = new ram.relationshiptree.EditorRelationshipTree({
    	id: "relationshipTree",
        model: treeModel,
        showRoot: false,
        persist: false,
        relationshipTypes: relationshipTypes
    },
    "relationshipTree");
}

function fetchAvailableRelationshipTypes() {
	dojo.xhrGet({
		url: getAbsoluteUrl(RELATIONSHIP_TYPES_URL),
		handleAs: "json",
		sync: true,
		load: loadAddRelationshipButton
	});
}

function loadAddRelationshipButton(response, ioArgs) {
	relationshipTypes = response;
	
	if (dijit.byId("RelationshipToolbar"))
		dijit.byId("RelationshipToolbar").destroyRecursive(false);
	if (dijit.byId("RelationshipTypesMenu"))
		dijit.byId("RelationshipTypesMenu").destroyRecursive(false);
	
	if (relationshipTypes === null)
		return;
	
	var pMenuBar = new dijit.Toolbar({id: "RelationshipToolbar", "class": "RelationshipToolbar"});
	
	var menu = new jazz.ui.Menu({
		id: "RelationshipTypesMenu",
        style: "display: none;"
    });
	
	for (var i=0; i<relationshipTypes.length; i++) {
	    var menuItem = new dijit.MenuItem({
	        label: relationshipTypes[i].name,
	        id: relationshipTypes[i].type
	    });
	    dojo.connect(menuItem.domNode, "onclick", function() {relationshipTypeToAdd = this.id});
	    dojo.connect(menuItem.domNode, "onclick", dojo.hitch(window, "openAssetSelectionDialog", addRelatedAssets));
	    
	    menu.addChild(menuItem);
	}
    
    var insertButton = new dijit.form.DropDownButton({
        label: "Add Relationship",
        iconClass: "dijitEditorIcon rowActionInsert",
        dropDown: menu,
        showLabel: true
    });
    pMenuBar.addChild(insertButton);
	
    dojo.place(pMenuBar.domNode, "addRelationshipButton");
}

function openAssetSelectionDialog(_callback) {
	var dialog = dijit.byId('ram_assetSelectionDialog');
	if (dialog)
		dialog.destroy();
	dialog = new ram.RamDialog({title: 'Add Related Assets', id: 'ram_assetSelectionDialog'});
	dojo.body().appendChild(dialog.domNode);
	dialog.startup();
	dialog.attr('content', '<div style="width: 640px; height: 480px;"></div>');
	// need to wait for the dialog to appear before loading the iframe, otherwise IE throws a fit (see defect 87844)
	dialog.onOpen = function() {
		var picker = dijit.byId('jazz_remoteChooserContainer');
		if (picker)
			picker.destroy();
		picker = new jazz.ui.RemoteChooserContainer({
			id: 'jazz_remoteChooserContainer',
			remoteLocation: getAbsoluteUrl('secure/delegated/assetSelection.faces'), 
			onData: _callback
		});
		dojo.style(picker.domNode, 'width', '640px');
		dojo.style(picker.domNode, 'height', '480px');
		dialog.attr('content', picker.domNode);
	};
	dialog.show();
}

function addRelatedAssets(json) {
	var response = dojo.fromJson(json);
	response.relationshipType = relationshipTypeToAdd;
	var results = response['oslc:results'];
	if (results.length > 0) {
		for (var i=0; i<results.length; i++) {
			dojo.xhrGet({
				url: results[i]['rdf:resource'],
				handleAs: 'json',
				headers: {
					'Accept': 'application/json',
					'OSLC-Core-Version': '2.0'
				},
				load: dojo.hitch(this, function(asset) {
					var tree = dijit.byId('relationshipTree');
					var newItem = {
							guid: asset['oslc_asset:guid'],
							version: asset['oslc_asset:version'], 
							name: asset['dcterms:title'], 
							assetTypeUri: unescape(asset['dcterms:type']['rdf:resource']), 
							jsObjType: 'asset', 
							relationshipInfo: {
								id: 0, 
								stateId: 0, 
								toAssetId: 0, 
								range: null, 
								source: false, 
								reverseRange: false
							}
						};
					tree.addRelatedAsset(relationshipTypeToAdd, newItem);
				}),
				// The ERROR function will be called in an error case.
				error: function(response, ioArgs) {
					console.error("HTTP status code: ", ioArgs.xhr.status);
					return response;
				}
			});
		}

		var dialog = dijit.byId('ram_assetSelectionDialog');
		if (dialog) {
			dialog.hide();
			dialog.destroy();
		}
	} else { 
		// dialog cancel button clicked
		var dialog = dijit.byId('ram_assetSelectionDialog');
		if (dialog) {
			dialog.hide();
			dialog.destroy();
		}
	}
}

function saveRelatedAssets() {
	if (relationshipTree!=null)
		relationshipTree.model.store.save();
}

function updateRelationshipInput(saveCompleteCallback, saveFailedCallback, newFileContentString) {
	convertDojoStoreToObjectModel(this._arrayOfAllItems[0]);
}

function convertDojoStoreToObjectModel(rootItem) {
	var r = new RelationshipList(rootItem);
	
	relationshipTree.model.getChildren(rootItem, dojo.hitch(this, function(children){
		dojo.forEach(children, dojo.hitch(this, addRelationship, r));
	}));
	
	var json = dojo.toJson(r);
	
	dojo.byId("assetUploadForm:relatedAssetsJson").value = json;
	
	saveCompleteCallback();
}

function addRelationship(root, item) {
	var rel = new Relationship(item);
	root.relationships.push(rel);
	
	relationshipTree.model.getChildren(item, dojo.hitch(this, function(children){
		dojo.forEach(children, dojo.hitch(this, addConstraint, rel));
	}));
}

function addConstraint(relationship, item) {
	var constraint = new Constraint(item);
	relationship.constraints.push(constraint);
	
	relationshipTree.model.getChildren(item, dojo.hitch(this, function(children){
		dojo.forEach(children, dojo.hitch(this, addConstraintChild, constraint));
	}));
}

function addConstraintChild(constraint, item) {
	var type = relationshipTree.model.store.getValue(item, "jsObjType");
	if (type=="constraint")
		addConstraint(constraint, item);
	else
		addAssetEntry(constraint, item);
}

function addAssetEntry(constraint, item) {
	var re = new RelationshipEntry(item);
	constraint.relatedAssets.push(re);
}

function uploadComplete() {
	dojo.byId("assetUploadForm:artifactFragment:uploadAjaxButton").click();
}

function displayArtifactRefreshStatusErrMsg() {
	var artifactRefreshStatus = dojo.byId("artifactRefreshStatus").innerHTML;
	if("done" != artifactRefreshStatus){
		dojo.byId("artifactRefreshStatusSpan").style.visibility = 'visible';
	}
}

function toggleAssetSubmitButton(enable) {
	if(enable){
		if(dojo.byId("assetUploadForm:saveButton")){
			dojo.byId("assetUploadForm:saveButton").removeAttribute("disabled"); // setAttribute("disabled","false") does not work
			dojo.byId("assetUploadForm:saveButton").setAttribute("class","commandExButton");
		}
		if(dojo.byId("assetUploadForm:topSaveButton")){
			dojo.byId("assetUploadForm:topSaveButton").removeAttribute("disabled"); // setAttribute("disabled","false") does not work
			dojo.byId("assetUploadForm:topSaveButton").setAttribute("class","commandExButton");
		}
		if(dojo.byId("assetUploadForm:finish_submit_update")){
			dojo.byId("assetUploadForm:finish_submit_update").removeAttribute("disabled"); // setAttribute("disabled","false") does not work
			dojo.byId("assetUploadForm:finish_submit_update").setAttribute("class","commandExButton");
		}
		if(dojo.byId("assetUploadForm:topFinish_submit_update")){
			dojo.byId("assetUploadForm:topFinish_submit_update").removeAttribute("disabled"); // setAttribute("disabled","false") does not work
			dojo.byId("assetUploadForm:topFinish_submit_update").setAttribute("class","commandExButton");
		}
	} else {
		if(dojo.byId("assetUploadForm:saveButton")){
			dojo.byId("assetUploadForm:saveButton").setAttribute("disabled","true");
			dojo.byId("assetUploadForm:saveButton").setAttribute("class","commandExButton_disabled");
		} 
		if(dojo.byId("assetUploadForm:topSaveButton")){
			dojo.byId("assetUploadForm:topSaveButton").setAttribute("disabled","true");
			dojo.byId("assetUploadForm:topSaveButton").setAttribute("class","commandExButton_disabled");
		}
		if(dojo.byId("assetUploadForm:finish_submit_update")){
			dojo.byId("assetUploadForm:finish_submit_update").setAttribute("disabled","true");
			dojo.byId("assetUploadForm:finish_submit_update").setAttribute("class","commandExButton_disabled");
		}
		if(dojo.byId("assetUploadForm:topFinish_submit_update")){
			dojo.byId("assetUploadForm:topFinish_submit_update").setAttribute("disabled","true");
			dojo.byId("assetUploadForm:topFinish_submit_update").setAttribute("class","commandExButton_disabled");
		}
	}
}

function fetchSuggestedRelationships() {
	dojo.xhrGet({
		url: getAbsoluteUrl(RELATIONSHIP_SUGGESTIONS_URL),
		handleAs: "json",
		sync: false,
		load: loadSuggestedRelationships
	});
}

function loadSuggestedRelationships(response, ioArgs) {
	var existing = dijit.byId("StyledBox_suggestedRelationships");
	if (existing) {
		existing.destroyRecursive(true);
		dojo.empty("suggestedRelationships");
	}
	if (response.length == 0)
		return;
	
	var table = dojo.create("table", {"class": "listTable", id: "Suggested_Relationships"});
	var tbody = dojo.create("tbody", null, table);
	dojo.forEach(response, function(data, i){
		data.resourceUri = getAbsoluteUrl(data.resourceUri);
		var row = dojo.create("tr", null, tbody);
		var cell1 = dojo.create("td", null, row);
		var navLink = new jazz.ui.ResourceLink({
			id: "Suggested_Related_Asset_" + i,
			uri: data.resourceUri, 
			presentationUri: data.resourceUri,
			lazyFetch: true,
			label: data.assetName
		});
		dojo.place(navLink.domNode, cell1);
		
		var cell2 = dojo.create("td", {width: "22px"}, row);
		var pMenuBar = new dijit.Toolbar({"class": "InlineToolbar"});
		var insertButton = new dijit.form.Button({
			id: "Suggested_Related_Asset_Button_" + i,
            label: "Add",
            iconClass: "dijitEditorIcon rowActionInsert",
            showLabel: false,
            onClick: dojo.hitch(data, addRelatedAsset)
        });
        pMenuBar.addChild(insertButton);
        pMenuBar.placeAt(cell2);
		
	});
	var countS = "("+response.length+")"
	var styledBox = new jazz.ui.StyledBox({id:"StyledBox_suggestedRelationships", primaryTitle: "Suggested Relationships", secondaryTitle: countS, trim: jazz.ui.styledBox.TrimStyles.BLUE, closable: false, contentNode: table});
	dojo.place(styledBox.domNode, "suggestedRelationships");
}

function addRelatedAsset() {
	dojo.xhrGet({
		url: this.resourceUri,
		handleAs: "json",
		headers: {"Accept": "application/json"},
		load: dojo.hitch(this, function() {
			var tree = dijit.byId("relationshipTree");
			var newItem = {"guid": arguments[0].id["ras:GUID"],"version": arguments[0].id["ras:version"], "name": arguments[0].title, "assetTypeUri": unescape(arguments[0].type.resource), "jsObjType":"asset", "relationshipInfo": {"id": 0, "stateId": 0, "toAssetId": 0, "range": null, "source": false, "reverseRange": false}};

			tree.addRelatedAsset("dependency", newItem);
		}),
		// The ERROR function will be called in an error case.
		error: function(response, ioArgs) {
			console.error("HTTP status code: ", ioArgs.xhr.status);
			return response;
		}
	});
}

function RelationshipList() {
	this.relationships = [];
}

function Relationship(item) {
	this.name = relationshipTree.model.store.getValue(item, "name");
	this.jsObjType = relationshipTree.model.store.getValue(item, "jsObjType");
	this.type = relationshipTree.model.store.getValue(item, "type");
	this.constraints = [];
}

function Constraint(item) {
	this.name =  relationshipTree.model.store.getValue(item, "name");
	this.relatedAssets = [];
	this.constraints = [];
	this.constraint = getNonUndefinedStringValue(relationshipTree.model.store.getValue(item, "constraint"));
	this.required = getNonUndefinedStringValue(relationshipTree.model.store.getValue(item, "required"));
	this.type = getNonUndefinedIntegerValue(relationshipTree.model.store.getValue(item, "type")); 
	this.minCount = getNonUndefinedIntegerValue(relationshipTree.model.store.getValue(item, "minCount")); 
	this.maxCount = getNonUndefinedIntegerValue(relationshipTree.model.store.getValue(item, "maxCount"));
	this.assetTypeUri = getNonUndefinedStringValue(relationshipTree.model.store.getValue(item, "assetTypeUri"));
	this.assetTypeName = getNonUndefinedStringValue(relationshipTree.model.store.getValue(item, "assetTypeName"));
	this.jsObjType = relationshipTree.model.store.getValue(item, "jsObjType"); 
	this.constraintString = getNonUndefinedStringValue(relationshipTree.model.store.getValue(item, "constraintString"));
}

function RelationshipEntry(item) {
	this.guid = relationshipTree.model.store.getValue(item, "guid");
	this.version = relationshipTree.model.store.getValue(item, "version");
	this.name = relationshipTree.model.store.getValue(item, "name");
	this.assetTypeUri = relationshipTree.model.store.getValue(item, "assetTypeUri");
	this.jsObjectType = relationshipTree.model.store.getValue(item, "jsObjType");
	this.relationshipInfo = relationshipTree.model.store.getValue(item, "relationshipInfo") ? relationshipTree.model.store.getValue(item, "relationshipInfo") : null;
}

function RelationshipInfo(item) {
	this.id = getNonUndefinedIntegerValue(item.id);
	this.reverseRange = item.reverseRange ? item.reverseRange : false; 
	this.source = item.source ? item.source : false;
	this.stateId = getNonUndefinedIntegerValue(item.stateId);
	this.toAssetId = getNonUndefinedIntegerValue(item.toAssetId);
	this.range = item.range ? new RelationshipRange(item.range) : null;
}

function RelationshipRange(item) {
	if (item) {
		this.lowestVersion = getNonUndefinedStringValue(item.lowestVersion);
		this.lowestVersionType = getNonUndefinedStringValue(item.lowestVersionType);
		this.highestVersion = getNonUndefinedStringValue(item.highestVersion);
		this.highestVersionType = getNonUndefinedStringValue(item.highestVersionType);
		this.type = getNonUndefinedStringValue(item.type);
	} else {
		this.lowestVersion = "";
		this.lowestVersionType = "";
		this.highestVersion = "";
		this.highestVersionType = "";
		this.type = "";
	}
}

/*******************************
 * Functions for validation and validation messages
 *******************************/
function getRelatedAssetsMessages() {
	dojo.xhrGet({
		url: getAbsoluteUrl(RELATED_MESSAGES_URL),
		handleAs: "json",
		
		// The LOAD function will be called on a successful response.
		load: function(response, ioArgs) {
			processMessages(response, "rel");
		}
	});
}
function getContentMessages() {
	dojo.xhrGet({
		url: getAbsoluteUrl(CONTENT_MESSAGES_URL),
		handleAs: "json",
		
		// The LOAD function will be called on a successful response.
		load: function(response, ioArgs) {
			processMessages(response, "content");
			processMessages(response, "content2");
		}
	});
}
function processMessages(messages, prefix) {
	var infos = 0;
	var errors = 0;
	
	var errorMsgTxt = "";
	var infoMsgTxt = "";
	
	for (var i=0; i<messages.length; i++) {
		if (messages[i].severity.severityName == "INFO") {
			if (infos>0) {
				infoMsgTxt += '<br/>';
			}
			infoMsgTxt += messages[i].summary;
			infos++;
		} else if (messages[i].severity.severityName == "ERROR") {
			if (errors>0) {
				errorMsgTxt += '<br/>';
			}
			errorMsgTxt += messages[i].summary;
			errors++;
		}
	}
	
	if (dojo.byId(prefix + 'Icon'))
		dojo.destroy(prefix + 'Icon');
	if (dijit.byId(prefix + 'ErrMsgs'))
		dijit.byId(prefix + 'ErrMsgs').destroy();
	if (dijit.byId(prefix + 'InfoMsgs'))
		dijit.byId(prefix + 'InfoMsgs').destroy();
	
	if (errors>0) {
		dojo.create("img", { id: prefix+"Icon", src: IMG_ERROR, "class": "iconPrefix"}, dojo.byId(prefix+"TabIcon"), "first");

		var errMsg = new ram.RamMessage({id:prefix+"ErrMsgs", message: errorMsgTxt, styleClass: "errorBlock"});
		var refNode = dojo.byId(prefix + 'AjaxMessages');
		if (refNode)
			dojo.place(errMsg.domNode, refNode, "last");
	} 
	if (infos>0) {
		if (errors==0) {
			dojo.create("img", { id: prefix+"Icon", src: IMG_INFO, "class": "iconPrefix"}, dojo.byId(prefix+"TabIcon"), "first");
		}
		var infoMsg = new ram.RamMessage({id:prefix+"InfoMsgs", message: infoMsgTxt, styleClass: "infoBlock"});
		var refNode = dojo.byId(prefix + 'AjaxMessages');
		if (refNode)
			dojo.place(infoMsg.domNode, refNode, "last");
	}
}

/*******************************
 * Functions for handling when the submit/update buttons are clicked
 *******************************/
function checkIfVersionChanged(thing, evt) {
	var versionChanged = document.getElementById("assetUploadForm:assetVersionInputText").value != document.getElementById("assetUploadForm:originVersion").value;
	if (versionChanged) {
		showNewVersionDialog();
	} else {
		showUpdateCommentDialog();
	}
}

function createDialog(dialogId, title, content, show, styleClass) {
	var applet = dojo.query('applet');
	if (applet)
		applet.style('visibility','hidden');
	var dialog = dijit.byId(dialogId);
	if (!dialog) {
		if (styleClass) {
			dialog = new ram.RamDialog({ id:dialogId, title: title, content: content, "class": styleClass});
		} else {
			dialog = new ram.RamDialog({ id:dialogId, title: title, content: content});
		}
		dojo.body().appendChild(dialog.domNode); 
		dialog.startup();
	} else {
		dialog.setContent(content);
	}
	
	if (show) {
		dialog.show();
	}
	
	return dialog;
}

// TODO this should be a global function
function getNonUndefinedStringValue(value) {
	return value ? value : "";
}
function getNonUndefinedIntegerValue(value) {
	return value ? value : 0;
}
